
#include "Arduino.h"
#include "Wire.h"
#include "I2Cdev.h"
#include "MPU6050.h"
#define SET_ACQUISITION 100
#define STOP_ACQUISITION 101
#define MAX_BUF_SIZE 210
#define INTERRUPT_PIN 2 
#define TTL_OUT 9 // sortie TTL
#define DIV_TTL_OUT 4 // diviseur de fréquence
#define TRIGGER_INPUT 3 // pour le déclenchement externe
// Arduino nano : A4=SDA, A5=SCL, D2=INT
    		
MPU6050 mpu;
int16_t ax, ay, az;
int16_t buffer[MAX_BUF_SIZE];
uint16_t indice_buffer;
volatile uint8_t data_ready;
uint8_t acquisition;
uint8_t gyro;
uint8_t mpuIntStatus;
uint8_t buffer_size; // doit être divisible par 6
int8_t compteur;
char flag;
uint16_t ct;
bool start;
uint16_t cnt;
    		 
void data_ready_interrupt() {
  data_ready = 1;
  ct += 1;
  cnt += 1;
  if (ct==DIV_TTL_OUT) {
    ct = 0;
    if (flag==1) {
      flag=0;
      digitalWrite(TTL_OUT,LOW);
    }
    else {
      flag = 1;
      digitalWrite(TTL_OUT,HIGH);
    }
  }
}    	   		  
    		  
void setup() {
  char c;
  Wire.begin();
  Serial.begin(19200);
  Serial.setTimeout(0);
  c = 0;
  Serial.write(c);
  c = 255;
  Serial.write(c);
  c = 0;
  Serial.write(c);
  indice_buffer = 0;
  data_ready = 0;
  acquisition = 0;
  mpu.setIntDataReadyEnabled(false);
  attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN),data_ready_interrupt,RISING);
  mpuIntStatus = mpu.getIntStatus();
  pinMode(TTL_OUT,OUTPUT);
  digitalWrite(TTL_OUT,LOW);
  flag = 0;
#ifdef TRIGGER_INPUT
  attachInterrupt(digitalPinToInterrupt(TRIGGER_INPUT),start_acquisition,RISING);
#endif
}    		   
    		   
void lecture_acquisition() {
    uint32_t c1,c2,c3,c4;
    uint8_t accel_range;
    uint8_t gyro_range;
    uint8_t rate_div;
    uint8_t dlpf_mode;
    
    while (Serial.available()<5) {};
    accel_range = Serial.read();
    gyro_range = Serial.read();
    rate_div = Serial.read();
    dlpf_mode = Serial.read();
    buffer_size = Serial.read();
    if (gyro_range<4) {gyro = 1;} else gyro = 0;
    mpu.initialize();
    mpu.setFullScaleAccelRange(accel_range);
    if (gyro) mpu.setFullScaleGyroRange(gyro_range);
    mpu.setDLPFMode(dlpf_mode);
    mpu.setRate(rate_div);
    data_ready = 0;
    acquisition = 0;
    indice_buffer = 0;
    compteur = 0;
    ct = 0;
#ifndef TRIGGER_INPUT
    start = true;
#endif
}    		     
    		     
#ifdef TRIGGER_INPUT
void start_acquisition() {
  if (acquisition==0) {
    start = true;
  }
}
#endif
    		     
void stop_acquisition() {
  acquisition = 0;
  mpu.setIntDataReadyEnabled(false);
}   		     
    		     
void loop() {
   char com;
   if (start) {
      start = false;
      cnt = 0;
      acquisition = 1;
      mpu.setIntDataReadyEnabled(true);
   }
   if ((compteur==0)&&Serial.available()>0) {
        com = Serial.read();
        if (com==SET_ACQUISITION) lecture_acquisition();
        if (com==STOP_ACQUISITION) stop_acquisition();
   }
   compteur += 1;
    if ((data_ready)&&(acquisition)) {
        data_ready = 0;
        delayMicroseconds(100);
        if (gyro) {
          mpu.getMotion6(&buffer[indice_buffer+1],&buffer[indice_buffer+2],&buffer[indice_buffer+3],&buffer[indice_buffer+4],&buffer[indice_buffer+5],&buffer[indice_buffer+6]);
          buffer[indice_buffer] = cnt;
          indice_buffer += 7;
        }
        else {
          mpu.getAcceleration(&buffer[indice_buffer+1],&buffer[indice_buffer+2],&buffer[indice_buffer+3]);
          buffer[indice_buffer] = cnt;
          indice_buffer += 4;
        }
        if (indice_buffer == buffer_size) {
            indice_buffer = 0;
            Serial.write((uint8_t *)buffer,buffer_size*2);
        }
    }
}   		      
    		      